
while [ $# -gt 0 ]; do
    case "$1" in
	-p) OPT_PLOT_ONLY=1; shift;;
	-X|--no-x11-plot) OPT_NO_X11=1; shift;;
    esac
done

#set -x
set -e

PATH=$PWD/../_compiled/bin/:$PATH

RTPRIO=${RTPRIO:-60}
COUNT=${COUNT:-10000}

boost_irq_prio() {
    local PRIO=${1:-50}
    CAN_IRQ=`sed -ne '/can/ s/:.*//p' /proc/interrupts`
    if [ -n "$CAN_IRQ" ]; then
	IRQ_PID=`pidof IRQ-${CAN_IRQ/ /}` || true
	if [ -n "$IRQ_PID" ]; then
	    schedtool -F -p $PRIO $IRQ_PID
	    return 0
	fi
    fi
    return 1
}

rmmod_can() {
    modprobe -r lincan kvaser_pci vcan
}

prepare_lincan_hw() {
    rmmod_can
    modprobe -C /dev/null lincan hw=pcican-q io=1 baudrate=1000,1000,1000,1000 processlocal=0
    CAN0=/dev/can0
    CAN1=/dev/can1
    DRIVER=lincan
    sleep 0.1
}

prepare_lincan_virtual() {
    rmmod_can
    modprobe -C /dev/null lincan hw=virtual io=0 baudrate=0
    CAN0=/dev/can0
    DRIVER=lincan-virtual
    sleep 0.1
}

prepare_socketcan_hw() {
    rmmod_can
    modprobe kvaser_pci
    echo 1000000 > /sys/class/net/can0/can_bittiming/bitrate
    ip link set up dev can0
    echo 1000000 > /sys/class/net/can1/can_bittiming/bitrate
    ip link set up dev can1
    echo 1000000 > /sys/class/net/can2/can_bittiming/bitrate
    ip link set up dev can2
    echo 1000000 > /sys/class/net/can3/can_bittiming/bitrate
    ip link set up dev can3
    CAN0=socketcan:can0
    CAN1=socketcan:can1
    DRIVER=socketcan
}

prepare_socketcan_virtual() {
    rmmod_can
    modprobe vcan
    ip link add dev vcan0 type vcan
    ip link set up dev vcan0
    CAN0=socketcan:vcan0
    DRIVER=socketcan-virtual
}

run_tests() {
    tests=${TESTS:-socketcan_hw lincan_hw}
    for tst in $tests; do
	boost_irq_prio 50 || true
	prepare_$tst
	echo "----------------------"
	t
    done
}

plot() {
    for i in "${!PLOT_CMD[@]}"; do
	cmd="${PLOT_CMD[$i]}";
	cmd=$(echo "$cmd" | sed -e "/set title/ s/[\"']\(.*\)[\"']/'\1 ($(uname -r), CPU\\\\@${clk}Mhz)'/")
	if [ -z "$OPT_NO_X11" ]; then
	    echo "set terminal x11 enhanced; ${cmd}" | gnuplot -persist
	fi
	I=${i/0/}
	echo "set terminal postscript color eps enhanced;
	      ${cmd}" | gnuplot | epstopdf --filter > `basename $0 .sh`$I.pdf
	mkdir -p thumb
	convert -geometry 1000x100 -gamma 0.3 `basename $0 .sh`$I.pdf thumb/`basename $0 .sh`$I.png
	convert -density 200 -gamma 0.7 -geometry 1000x500 `basename $0 .sh`$I.pdf `basename $0 .sh`$I.png
    done
}

create_dirs_and_links() {
    local test=$1
    local kver=$2
    local clck=$3

    local  d=results/by-test/$test/$kver/$clck
    mkdir -p $d
    mkdir -p results/by-test/$test/$clck # 1
    mkdir -p results/by-kern/$kver/$clck # 2
    mkdir -p results/by-kern/$kver/$test # 3
    mkdir -p results/by-clck/$clck/$kver # 4
    mkdir -p results/by-clck/$clck/$test # 5
    ln -sfT ../../../${d#results/} results/by-test/$test/$clck/$kver # 1
    ln -sfT ../../../${d#results/} results/by-kern/$kver/$clck/$test # 2
    ln -sfT ../../../${d#results/} results/by-kern/$kver/$test/$clck # 3
    ln -sfT ../../../${d#results/} results/by-clck/$clck/$kver/$test # 4
    ln -sfT ../../../${d#results/} results/by-clck/$clck/$test/$kver # 5
    echo $d
}

go() {
    echo "####################"
    mhz=${MHZ:-2400 300}
    for clk in $mhz; do
    echo "===================="
	dir=$(create_dirs_and_links `basename $0 .sh` `uname -r` $clk)
	set -x
	pushd $dir

	if [ ! -n "$OPT_PLOT_ONLY" ]; then
    	    cpufreq-set -f ${clk}MHz
	    run_tests
    	    cpufreq-set -f 2400MHz
	fi
	plot
	popd
	set +x
    done

}

trap 'go' EXIT
